94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
145 lines
4.7 KiB
Python
145 lines
4.7 KiB
Python
from typing import List, Optional
|
|
|
|
import torch
|
|
import triton
|
|
import triton.language as tl
|
|
|
|
|
|
def transform_index_page_table_prefill(**kwargs):
|
|
return transform_index_page_table_prefill_ref(**kwargs)
|
|
|
|
|
|
def transform_index_page_table_decode(**kwargs):
|
|
return transform_index_page_table_decode_ref(**kwargs)
|
|
|
|
|
|
@triton.jit
|
|
def transform_index_page_table_decode_kernel(
|
|
page_table_ptr: torch.Tensor,
|
|
topk_indices_ptr: torch.Tensor,
|
|
result_ptr: torch.Tensor,
|
|
page_size: tl.constexpr,
|
|
max_seqlen_k: tl.constexpr,
|
|
):
|
|
TOPK: tl.constexpr = 2048
|
|
req_id = tl.program_id(0)
|
|
page_table_ptr = page_table_ptr + req_id * max_seqlen_k
|
|
topk_indices_ptr = topk_indices_ptr + req_id * TOPK
|
|
result_ptr = result_ptr + req_id * TOPK
|
|
|
|
offset = tl.arange(0, TOPK) # topk should be 2048
|
|
loaded_topk_indices = tl.load(topk_indices_ptr + offset)
|
|
mask = loaded_topk_indices >= 0
|
|
loaded_kv_indices = tl.load(page_table_ptr + loaded_topk_indices, mask=mask)
|
|
tl.store(result_ptr + offset, loaded_kv_indices, mask=mask)
|
|
tl.store(result_ptr + offset, -1, mask=~mask)
|
|
|
|
|
|
def transform_index_page_table_decode_fast(
|
|
page_table: torch.Tensor,
|
|
topk_indices: torch.Tensor,
|
|
result: Optional[torch.Tensor] = None,
|
|
page_size: int = 1,
|
|
) -> torch.Tensor:
|
|
"""
|
|
Transform the page table according to topk indices for sparse topk attention.
|
|
Args:
|
|
page_table: [qo_len, max_seqlen_k], the original page table
|
|
topk_indices: [qo_len, topk], the topk indices for each query position
|
|
Returns:
|
|
transformed_page_table: [qo_len, topk], the transformed page table
|
|
For out-of-bound indices in topk_indices, this should be filled with -1.
|
|
"""
|
|
assert page_size == 1
|
|
assert page_table.shape[0] == topk_indices.shape[0]
|
|
assert topk_indices.shape[1] == 2048
|
|
qo_len = topk_indices.shape[0]
|
|
max_seqlen_k = page_table.shape[1]
|
|
if result is None:
|
|
result = torch.empty_like(topk_indices, dtype=torch.int32)
|
|
# Launch triton kernel
|
|
grid = (qo_len,)
|
|
transform_index_page_table_decode_kernel[grid](
|
|
page_table,
|
|
topk_indices,
|
|
result,
|
|
page_size,
|
|
max_seqlen_k=max_seqlen_k,
|
|
)
|
|
return result
|
|
|
|
|
|
def transform_index_page_table_prefill_fast(
|
|
page_table: torch.Tensor,
|
|
topk_indices: torch.Tensor,
|
|
extend_lens_cpu: List[int],
|
|
page_size: int = 1,
|
|
) -> torch.Tensor:
|
|
# TODO(baizhou): can be implemented with another triton kernel
|
|
assert page_size == 1
|
|
result = torch.empty_like(topk_indices, dtype=torch.int32)
|
|
assert len(extend_lens_cpu) == page_table.shape[0]
|
|
offset = 0
|
|
for i, l in enumerate(extend_lens_cpu):
|
|
transform_index_page_table_decode_fast(
|
|
page_table[i].unsqueeze(0).expand(l, -1),
|
|
topk_indices[offset : offset + l],
|
|
result=result[offset : offset + l],
|
|
)
|
|
offset += l
|
|
assert offset == topk_indices.shape[0]
|
|
return result
|
|
|
|
|
|
def transform_index_page_table_decode_ref(
|
|
page_table: torch.Tensor,
|
|
topk_indices: torch.Tensor,
|
|
result: Optional[torch.Tensor] = None,
|
|
page_size: int = 1,
|
|
) -> torch.Tensor:
|
|
assert page_size == 1
|
|
assert page_table.shape[0] == topk_indices.shape[0]
|
|
if result is None:
|
|
result = torch.empty_like(topk_indices, dtype=torch.int32)
|
|
assert result.shape == topk_indices.shape
|
|
torch.gather(
|
|
page_table.to(result.dtype),
|
|
dim=1,
|
|
index=topk_indices.clamp(min=0),
|
|
out=result,
|
|
)
|
|
result[topk_indices < 0] = -1
|
|
return result
|
|
|
|
|
|
def transform_index_page_table_prefill_ref(
|
|
page_table: torch.Tensor,
|
|
topk_indices: torch.Tensor,
|
|
extend_lens_cpu: List[int],
|
|
page_size: int = 1,
|
|
) -> torch.Tensor:
|
|
assert page_size == 1
|
|
result = torch.empty_like(topk_indices, dtype=torch.int32)
|
|
assert len(extend_lens_cpu) == page_table.shape[0]
|
|
offset = 0
|
|
for i, l in enumerate(extend_lens_cpu):
|
|
transform_index_page_table_decode_ref(
|
|
page_table[i].unsqueeze(0).expand(l, -1),
|
|
topk_indices[offset : offset + l],
|
|
result=result[offset : offset + l],
|
|
)
|
|
offset += l
|
|
assert offset == topk_indices.shape[0]
|
|
return result
|
|
|
|
|
|
if __name__ == "__main__":
|
|
bs, topk, max_seqlen = 10, 2048, 3000
|
|
page_table = torch.randint(0, 100, (bs, max_seqlen), device="cuda")
|
|
topk_indices = torch.full((bs, topk), -1, device="cuda")
|
|
topk_indices[:, :1600] = torch.arange(1600).unsqueeze(0).repeat(bs, 1)
|
|
ref_result = transform_index_page_table_decode_ref(page_table, topk_indices)
|
|
result = transform_index_page_table_decode_fast(page_table, topk_indices)
|
|
assert torch.all(result == ref_result)
|
|
print("Passed")
|